@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -128,14 +128,18 @@
  * The keyword given must match one defined in the server configuration file.
  * DACS authentication is authoritative; if it fails, no other access control
  * scheme will be tried.
+ *
+ * To follow Apache's recent conventions, this module should probably
+ * be called mod_authnz_dacs since it can act as both an authentication and
+ * authorization provider.
  */
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2013\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: mod_auth_dacs.c 2582 2012-03-13 18:51:52Z brachman $";
+  "$Id: mod_auth_dacs.c 2646 2013-02-27 20:10:40Z brachman $";
 #endif
 
 #include "apr_strings.h"
@@ -157,6 +161,27 @@ static const char revid[] =
 #include "dacs_version.h"
 #endif
 
+/*
+ * XXX These make some assumptions about compatibility between versions
+ * that may need to be corrected in future releases of Apache.
+ */
+#if (AP_SERVER_MAJORVERSION_NUMBER == 2 && AP_SERVER_MINORVERSION_NUMBER >= 4)\
+  || AP_SERVER_MAJORVERSION_NUMBER >= 3
+#define APACHE_2_4_OR_NEWER
+static int is_apache_2_4_or_newer = 1;
+#endif
+
+#if (AP_SERVER_MAJORVERSION_NUMBER == 2 && AP_SERVER_MINORVERSION_NUMBER >= 2)\
+  || AP_SERVER_MAJORVERSION_NUMBER >= 3
+#define APACHE_2_2_OR_NEWER
+static int is_apache_2_2_or_newer = 1;
+#endif
+
+#ifdef APACHE_2_4_OR_NEWER
+#include "ap_provider.h"
+#include "mod_auth.h"
+#endif
+
 /*
  * Define DACS_DEBUG depending on how you want debugging output enabled.
  * Set it to 0 to omit debugging log messages, or 1 to produce them.
@@ -199,28 +224,29 @@ static APR_OPTIONAL_FN_TYPE(ssl_is_https) *rewrite_is_https = NULL;
 
 enum {
   /* Emit the ARGS_TRUNCATED_ARG before the SERVICE_ARGS variable */
-  DACS_SERVICE_ARGS_TRUNCATED_ARG   =  0,
-  DACS_SERVICE_ARGS 			    =  1,
-  DACS_SERVICE_AUTHORIZATION_ARG    =  2,
-  DACS_SERVICE_CONTENT_ENCODING_ARG =  3,
-  DACS_SERVICE_CONTENT_LENGTH_ARG   =  4,
-  DACS_SERVICE_CONTENT_TYPE_ARG	    =  5,
-  DACS_SERVICE_COOKIE_ARG 		    =  6,
-  DACS_SERVICE_FILENAME_ARG		    =  7,
-  DACS_SERVICE_HOSTNAME_ARG         =  8,
-  DACS_SERVICE_HTTPS_ARG            =  9,
-  DACS_SERVICE_METHOD_ARG		    = 10,
-  DACS_SERVICE_PATH_INFO_ARG        = 11,
-  DACS_SERVICE_POSTDATA_ARG         = 12,
-  DACS_SERVICE_PROXYREQ_ARG         = 13,
-  DACS_SERVICE_QUERY_ARG		    = 14,
-  DACS_SERVICE_REMOTE_ADDR_ARG	    = 15,
-  DACS_SERVICE_REMOTE_HOST_ARG	    = 16,
-  DACS_SERVICE_SERVER_PORT_ARG      = 17,
-  DACS_SERVICE_URI_ARG 			    = 18,
-  DACS_SERVICE_USER_AGENT_ARG       = 19,
-  DACS_PROXY_AUTH_ARG               = 20,
-  DACS_SERVICE_NARGS			    = (DACS_PROXY_AUTH_ARG + 1)
+  DACS_SERVICE_ARGS_TRUNCATED_ARG      =  0,
+  DACS_SERVICE_ARGS 			       =  1,
+  DACS_SERVICE_AUTHORIZATION_ARG       =  2,
+  DACS_SERVICE_CONTENT_ENCODING_ARG    =  3,
+  DACS_SERVICE_CONTENT_LENGTH_ARG      =  4,
+  DACS_SERVICE_CONTENT_TYPE_ARG	       =  5,
+  DACS_SERVICE_COOKIE_ARG 		       =  6,
+  DACS_SERVICE_FILENAME_ARG		       =  7,
+  DACS_SERVICE_HOSTNAME_ARG            =  8,
+  DACS_SERVICE_HTTPS_ARG               =  9,
+  DACS_SERVICE_METHOD_ARG		       = 10,
+  DACS_SERVICE_PATH_INFO_ARG           = 11,
+  DACS_SERVICE_POSTDATA_ARG            = 12,
+  DACS_SERVICE_PROXYREQ_ARG            = 13,
+  DACS_SERVICE_QUERY_ARG		       = 14,
+  DACS_SERVICE_REMOTE_ADDR_ARG	       = 15,
+  DACS_SERVICE_REMOTE_HOST_ARG	       = 16,
+  DACS_SERVICE_SERVER_PORT_ARG         = 17,
+  DACS_SERVICE_URI_ARG 			       = 18,
+  DACS_SERVICE_USER_AGENT_ARG          = 19,
+  DACS_SERVICE_PROXY_AUTHORIZATION_ARG = 20,
+  DACS_PROXY_AUTH_ARG                  = 21,
+  DACS_SERVICE_NARGS			       = (DACS_PROXY_AUTH_ARG + 1)
 };
 
 enum {
@@ -288,8 +314,7 @@ config_src(cmd_parms *c)
   if (s->defn_name == NULL)
 	return("");
 
-  return(apr_psprintf(c->temp_pool, " at %s:%d",
-					  s->defn_name, s->defn_line_number));
+  return(apr_psprintf(c->temp_pool, " at %s:%d", s->defn_name, s->defn_line_number));
 }
 
 static int cgi_pre_handler(request_rec *r);
@@ -461,8 +486,8 @@ dacs_merge_server_config(apr_pool_t *p, void *base_conf, void *vh_conf)
  * to the server configuration.
  */
 static const char *
-add_auth_dacs(cmd_parms *cmd, void *mconfig,
-			  const char *keyword, const char *path, const char *args)
+add_auth_dacs(cmd_parms *cmd, void *mconfig, const char *keyword, const char *path,
+			  const char *args)
 {
   dacs_server_config *sc_rec;
 
@@ -565,8 +590,7 @@ set_buf_size(cmd_parms *cmd, void *mconfig, const char *value)
  * authentication method in the server configuration.
  */
 static const char *
-set_dacs_method(cmd_parms *cmd, void *mconfig,
-				const char *keyword, const char *method)
+set_dacs_method(cmd_parms *cmd, void *mconfig, const char *keyword, const char *method)
 {
   dacs_server_config *sc_rec;
 
@@ -587,8 +611,7 @@ set_dacs_method(cmd_parms *cmd, void *mconfig,
  * the DACS configuration file method in the server configuration.
  */
 static const char *
-set_dacs_conf(cmd_parms *cmd, void *mconfig,
-			  const char *keyword, const char *path)
+set_dacs_conf(cmd_parms *cmd, void *mconfig, const char *keyword, const char *path)
 {
   dacs_server_config *sc_rec;
 
@@ -615,8 +638,7 @@ set_dacs_conf(cmd_parms *cmd, void *mconfig,
  * the DACS site configuration file method in the server configuration.
  */
 static const char *
-set_dacs_site_conf(cmd_parms *cmd, void *mconfig,
-				   const char *keyword, const char *path)
+set_dacs_site_conf(cmd_parms *cmd, void *mconfig, const char *keyword, const char *path)
 {
   dacs_server_config *sc_rec;
 
@@ -849,6 +871,7 @@ lookup_header(const request_rec *r, const char *name)
   return(NULL);
 }
 
+#ifdef NOTDEF
 static char *
 list_headers(const request_rec *r, const apr_table_t *t)
 {
@@ -863,12 +886,13 @@ list_headers(const request_rec *r, const apr_table_t *t)
   list = "";
   for (i = 0; i < hdrs_arr->nelts; ++i) {
 	if (hdrs[i].key != NULL)
-	  list = apr_pstrcat(r->pool, list, *list == '\0' ? "" : ",",
+	  list = apr_pstrcat(r->pool, list, (*list == '\0') ? "" : ",",
 						 hdrs[i].key, NULL);
   }
 
   return(list);
 }
+#endif
 
 static void
 acs_control(request_rec *r, dacs_buf_config *bc_rec, int *http_status,
@@ -911,12 +935,12 @@ acs_control(request_rec *r, dacs_buf_config *bc_rec, int *http_status,
 					   "acs_control: unset DACS_ACS");
 		  ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
 					   " and: args=\"%s\", unparsed_uri=\"%s\"",
-					   r->args == NULL ? "" : r->args,
-					   r->unparsed_uri == NULL ? "" : r->unparsed_uri);
+					   (r->args == NULL) ? "" : r->args,
+					   (r->unparsed_uri == NULL) ? "" : r->unparsed_uri);
 		  ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
 					   " and: parsed_uri->query=\"%s\" the_request=\"%s\"",
-					   r->parsed_uri.query == NULL ? "" : r->parsed_uri.query,
-					   r->the_request == NULL ? "" : r->the_request);
+					   (r->parsed_uri.query == NULL) ? "" : r->parsed_uri.query,
+					   (r->the_request == NULL) ? "" : r->the_request);
 		}
 		p += 16;
 		if (r->args != NULL)
@@ -946,11 +970,11 @@ acs_control(request_rec *r, dacs_buf_config *bc_rec, int *http_status,
 		if (debug_enabled(sc_rec)) {
 		  ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
 					   "acs_control: args=\"%s\", unparsed_uri=\"%s\"",
-					   r->args == NULL ? "" : r->args,
-					   r->unparsed_uri == NULL ? "" : r->unparsed_uri);
+					   (r->args == NULL) ? "" : r->args,
+					   (r->unparsed_uri == NULL) ? "" : r->unparsed_uri);
 		  ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
 					   " and: parsed_uri->query=\"%s\"",
-					   r->parsed_uri.query == NULL ? "" : r->parsed_uri.query);
+					   (r->parsed_uri.query == NULL) ? "" : r->parsed_uri.query);
 		}
 	  }
 	  else if (strncmp(p, "=Set-Header=", 12) == 0) {
@@ -959,7 +983,7 @@ acs_control(request_rec *r, dacs_buf_config *bc_rec, int *http_status,
 		*q = '\0';
 		if ((s = strchr(p + 12, ':')) != NULL && *(s + 1) == ' ') {
 		  *s = '\0';
-		  /* This will replace any existing header */
+		  /* This will replace any existing header. */
 		  apr_table_set(r->err_headers_out, p + 12, s + 2);
 		  if (debug_enabled(sc_rec))
 			ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
@@ -1292,9 +1316,15 @@ exec_dacs_acs(request_rec *r, const char *extpath, const char *extargs,
 	ext_args[DACS_SERVICE_REMOTE_HOST_ARG]
 	  = apr_psprintf(r->pool, "SERVICE_REMOTE_HOST=\"%s\"", remote_host);
 
+#ifdef APACHE_2_4_OR_NEWER
+  if (c->client_ip)
+	ext_args[DACS_SERVICE_REMOTE_ADDR_ARG]
+	  = apr_psprintf(r->pool, "SERVICE_REMOTE_ADDR=\"%s\"", c->client_ip);
+#else
   if (c->remote_ip)
 	ext_args[DACS_SERVICE_REMOTE_ADDR_ARG]
 	  = apr_psprintf(r->pool, "SERVICE_REMOTE_ADDR=\"%s\"", c->remote_ip);
+#endif
 
   if (r->hostname)
 	ext_args[DACS_SERVICE_HOSTNAME_ARG]
@@ -1329,6 +1359,10 @@ exec_dacs_acs(request_rec *r, const char *extpath, const char *extargs,
 	ext_args[DACS_SERVICE_AUTHORIZATION_ARG]
 	  = apr_psprintf(r->pool, "SERVICE_AUTHORIZATION='%s'", cp);
 
+  if ((cp = apr_table_get(r->headers_in, "Proxy-Authorization")) != NULL)
+	ext_args[DACS_SERVICE_PROXY_AUTHORIZATION_ARG]
+	  = apr_psprintf(r->pool, "SERVICE_PROXY_AUTHORIZATION='%s'", cp);
+
   /*
    * When proxying, ACS may be called twice: once for the original service
    * request and once for the proxy_exec program that sets up and executes
@@ -1727,6 +1761,8 @@ ap_log_error(APLOG_MARK, APLOG_INFO, 0, r->server,
   return(rc);
 }
 
+static int debug_sleep_secs = 15;
+
 /*
  * Check for both user authentication and authorization
  */
@@ -1753,7 +1789,7 @@ dacs_check_auth(request_rec *r)
 				 r->uri, c_uri);
   }
 
-  /* Extract which external was chosen */
+  /* Extract which external was chosen. */
   extname = dc_rec->dacs_extname;
 
   /* Check if we are supposed to handle this authentication. */
@@ -1786,7 +1822,7 @@ dacs_check_auth(request_rec *r)
   else
 	http_status = HTTP_OK;
 
-  /* Process control directives sent by DACS ACS */
+  /* Process control directives sent by dacs_acs. */
   bc_rec = get_dacs_buf_config(r);
   check_only = 0;
   do_zap = 0;
@@ -1808,7 +1844,7 @@ dacs_check_auth(request_rec *r)
 
 		/* Zap the newline. */
 		msg[len - 1] = '\0';
-		/* See set_error_document() */
+		/* See set_error_document(). */
 		if (ap_strchr_c(msg, ' '))
 		  type = "message";
 		else if (msg[0] == '/')
@@ -1905,7 +1941,7 @@ dacs_check_auth(request_rec *r)
   ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
    "mod_auth_dacs: %s [%s]: Failed: http_status=%d,r->status=%d,user=\"%s\"",
 			   extname, extpath, http_status, r->status,
-			   r->user != NULL ? r->user : "UNKNOWN");
+			   (r->user != NULL) ? r->user : "UNKNOWN");
 
   /*
    * Note that returning HTTP_UNAUTHORIZED results in the user being
@@ -1923,8 +1959,7 @@ dacs_check_user_id(request_rec *r)
 {
   char *extname;
   dacs_dir_config *dc_rec
-	= (dacs_dir_config *) ap_get_module_config(r->per_dir_config,
-											   &auth_dacs_module);
+	= (dacs_dir_config *) ap_get_module_config(r->per_dir_config, &auth_dacs_module);
   dacs_server_config *sc_rec
 	= (dacs_server_config *)
 	ap_get_module_config(r->server->module_config, &auth_dacs_module);
@@ -1934,14 +1969,14 @@ dacs_check_user_id(request_rec *r)
 
 	ap_getparents(c_uri);
 	ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-		 "mod_auth_dacs: dacs_check_user_id: uri=\"%s\" (\"%s\"), file=\"%s\"",
+				 "mod_auth_dacs: dacs_check_user_id: uri=\"%s\" (\"%s\"), file=\"%s\"",
 				 r->uri, c_uri, r->filename);
   }
 
-  /* Extract which external was chosen */
+  /* Extract which external was chosen. */
   extname = dc_rec->dacs_extname;
 
-  /* Check if we are supposed to handle this authentication */
+  /* Check if we are supposed to handle this authentication. */
   if (!extname)
 	return(DECLINED);
 
@@ -1950,7 +1985,7 @@ dacs_check_user_id(request_rec *r)
 
 static int
 dacs_init_handler(apr_pool_t *p, apr_pool_t *plog, apr_pool_t *ptemp,
-                             server_rec *s)
+				  server_rec *s)
 {
   char *str;
   dacs_server_config *sc_rec
@@ -1963,10 +1998,10 @@ dacs_init_handler(apr_pool_t *p, apr_pool_t *plog, apr_pool_t *ptemp,
 					   DACS_VERSION_RELEASE, DACS_VERSION_DATE);
   else
 	str = apr_psprintf(p, "mod_auth_dacs/%s", DACS_VERSION_RELEASE);
+  ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, "Using %s", str);
   ap_add_version_component(p, str);
 #else
 #if DACS_VERSION_NUMBER != 0
-
   ap_add_version_component(p, "+mod_auth_dacs");
 #endif
 
@@ -1979,24 +2014,115 @@ dacs_init_handler(apr_pool_t *p, apr_pool_t *plog, apr_pool_t *ptemp,
   return(OK);
 }
 
+#ifdef APACHE_2_4_OR_NEWER
+static const char *
+dacs_parse_config(cmd_parms *cmd, const char *require_line,
+				  const void **parsed_require_line)
+{
+  const char *t, *w;
+  int count = 0;
+  apr_pool_t *ptemp = cmd->temp_pool;
+
+  t = require_line;
+  while ((w = ap_getword_conf(ptemp, &t)) && w[0]) {
+	fprintf(stderr, "mod_auth_dacs Require: \"%s\"\n", w);
+	/*
+	 */
+	count++;
+  }
+
+#ifdef NOTDEF
+  if (count != 0)
+	return("'require dacs' takes no arguments");
+#endif
+
+  return(NULL);
+}
+
+static authz_status
+dacs_check_authorization(request_rec *r, const char *require_line,
+						 const void *parsed_require_line)
+{
+  int st;
+#ifdef NOTDEF
+  /* apr_ipsubnet_test should accept const but doesn't */
+  apr_ipsubnet_t **ip = (apr_ipsubnet_t **)parsed_require_line;
+
+  while (*ip) {
+	if (apr_ipsubnet_test(*ip, r->useragent_addr))
+	  return AUTHZ_GRANTED;
+	ip++;
+  }
+
+  /* authz_core will log the require line and the result at DEBUG */
+  return(AUTHZ_DENIED);
+#endif
+
+  st = dacs_check_auth(r);
+
+#ifdef APACHE_2_4_OR_NEWER
+  if (st == HTTP_PROXY_AUTHENTICATION_REQUIRED) {
+	r->status = st;
+	r->user = "";
+	return(AUTHZ_DENIED_NO_USER);
+  }
+#endif
+
+  if (st == HTTP_OK || st == OK)
+	return(AUTHZ_GRANTED);
+
+  return(AUTHZ_DENIED);
+}
+
+static const authz_provider authz_dacs_provider =
+{
+  &dacs_check_authorization,
+  NULL,
+};
+#endif
+
 static void
 dacs_register_hooks(apr_pool_t *p)
 {
+#ifdef APACHE_2_4_OR_NEWER
+  /*
+   * Make the old provider name an alias for "dacs-authz" to avoid having
+   * to modify Apache config files (specifically, "Require dacs" is
+   * equivalent to "Require dacs-authz").
+   *
+   * "If your module requires the old behavior and must perform access control checks
+   * on every sub-request with a different URI from the initial request, even if that
+   * URI matches the same set of access control configuration directives, then use
+   * AP_AUTH_INTERNAL_PER_URI."
+   */
+  ap_register_auth_provider(p, AUTHZ_PROVIDER_GROUP, "dacs", AUTHZ_PROVIDER_VERSION,
+							&authz_dacs_provider, AP_AUTH_INTERNAL_PER_URI);
+  ap_register_auth_provider(p, AUTHZ_PROVIDER_GROUP, "dacs-authz", AUTHZ_PROVIDER_VERSION,
+							&authz_dacs_provider, AP_AUTH_INTERNAL_PER_URI);
+
+  ap_hook_check_authn(dacs_check_user_id, NULL, NULL, APR_HOOK_MIDDLE,
+					  AP_AUTH_INTERNAL_PER_URI);
+
+  ap_hook_post_config(dacs_init_handler, NULL, NULL, APR_HOOK_MIDDLE);
+#else
   const char *const *aszPost;
   const char *const *aszPre;
 
-  if (strcmp(AP_SERVER_MINORREVISION, "2.2") == 0) {
+#ifdef APACHE_2_2_OR_NEWER
+  {
 	static const char *const post[] = { "mod_authz_user.c", NULL };
 
 	aszPre = NULL;
 	aszPost = post;
   }
-  else {
+#else
+  {
 	static const char *const post[] = { "mod_access.c", "mod_auth.c", NULL };
 
 	aszPre = NULL;
 	aszPost = post;
   }
+#endif
 
   ap_hook_post_config(dacs_init_handler, NULL, NULL, APR_HOOK_MIDDLE);
 
@@ -2008,6 +2134,7 @@ dacs_register_hooks(apr_pool_t *p)
 
   /* "Determine whether the identified remote user is permitted access" */
   ap_hook_auth_checker(dacs_check_auth, aszPre, aszPost, APR_HOOK_MIDDLE);
+#endif
 }
 
 module AP_MODULE_DECLARE_DATA auth_dacs_module = {
