@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2011
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: acs.h 2528 2011-09-23 21:54:05Z brachman $
+ * $Id: acs.h 2646 2013-02-27 20:10:40Z brachman $
  */
 
 /*****************************************************************************
@@ -229,6 +229,7 @@ typedef enum {
 } Acs_denial_reason;
 
 typedef struct Acs_environment {
+  Uri *uri;
   char *url;			/* This is accessed frequently; also appears in env */
   int do_eval;			/* If 0, parse but do not evaluate anything */
   Acl_rule *acl_rule;	/* The ACL applicable to the request, if any */
@@ -505,6 +506,31 @@ init_grant_attrs(Grant_attrs *a)
   a->permit_caching = NULL;
 }
 
+typedef enum {
+  URI_PATTERN_HOST_DN          = 0,
+  URI_PATTERN_HOST_DN_STAR     = 1,
+  URI_PATTERN_HOST_DN_PLUS     = 2,
+  URI_PATTERN_HOST_ANY         = 3,
+  URI_PATTERN_HOST_IP          = 4,
+  URI_PATTERN_HOST_INVALID     = -1,
+} Uri_pattern_host_type;
+
+typedef struct Uri_pattern {
+  char *uri;				/* URI pattern that was parsed */
+  int need_ssl;				/* 0==no SSL, 1==SSL, 2==either */
+  char *scheme;				/* Canonicalized to lowercase */
+  char *authority;
+  char *userinfo;
+  char *host;				/* Canonicalized to lowercase */
+  Uri_pattern_host_type host_pat_type;
+  struct in_addr host_addr;
+  struct in_addr host_mask;
+  in_port_t port;			/* Port number, if not 0 */
+  char *port_given;			/* Port string as given in the URI, if any */
+  char *path;				/* Decoded from the original URI */
+  Dsvec *path_parts;		/* If non-NULL, components of original URI path */
+} Uri_pattern;
+
 #include "rlink.h"
 
 #ifdef __cplusplus
@@ -609,6 +635,10 @@ extern int is_valid_id_attr(Dsvec *dsv, char *id);
 
 extern Value *init_value(Value *v, Token token, ...);
 
+extern Uri_pattern *uri_parse_pattern(char *str);
+extern int uri_pattern_match(Uri *uri, Uri_pattern *pat);
+extern int uri_pattern_match_selftest(void);
+
 #ifdef __cplusplus
 }
 #endif
