@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2011
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: local.h 2585 2012-03-15 16:20:19Z brachman $
+ * $Id: local.h 2646 2013-02-27 20:10:40Z brachman $
  */
 
 /*****************************************************************************
@@ -592,13 +592,15 @@ extern int dacs_init_allow_dups_default;
 extern Emit_format emit_format_default;
 extern char *emit_html_attrname_color;
 extern int emit_content_type_header;
+extern int dacs_no_conf_flag;
 extern int dacs_saw_command_line_log_level;
 extern int dacs_saw_command_line_format;
 extern char *dacs_compatibility_mode;
 extern char *dacs_version_number;
 extern char *dacs_version_release;
-extern int ssl_verify;
-extern int use_ssl;
+extern int dacs_enable_rule_patterns;
+extern int ssl_verify; extern int use_ssl;
+extern int forward_proxy_jurisdiction;
 extern char *dacs_log_format_default;
 extern char *dacs_log_format;
 
@@ -730,6 +732,8 @@ extern char *current_uri_sa(Ds *ods);
 extern char *current_uri_prog(void);
 extern char *current_uri_script(void);
 extern char *current_uri_pqf(char **path, char **query, char **fragment);
+extern char *set_dacs_service_uri(char *uri);
+extern int current_uri_is_proxied(char *uri);
 
 extern int is_internet_explorer(char *user_agent);
 extern char *ul_str(unsigned long val);
@@ -743,9 +747,12 @@ extern int is_ip_addr(const char *str, struct in_addr *addr);
 extern int parse_ip_expr(char *ip_expr, struct in_addr *addrp,
 						 struct in_addr *maskp);
 extern int match_ip_domain_name(const char *ip, char *domain_name);
+extern int match_domain_name_ip(char *domain_name, struct in_addr match_addr,
+								struct in_addr match_mask);
 extern int match_ip(const char *ip, struct in_addr match_addr,
 					struct in_addr match_mask);
-extern int match_domain_names(const char *subdomain, const char *domain);
+extern int match_domain_names(char *dn1, char *dn2);
+extern int match_subdomain_name(const char *subdomain, const char *domain);
 extern int match_ip_addr_range(char *addr_pattern, char *remote_addr);
 extern int is_from_address(char *address, char *remote_addr, char *remote_host,
 						   char **errmsg);
@@ -821,7 +828,7 @@ extern int dacstransform_main(int argc, char **argv, int do_init,
 							  void *main_out);
 extern int dacsversion_main(int argc, char **argv, int do_init, void *main_out);
 extern int dacsvfs_main(int argc, char **argv, int do_init, void *main_out);
-extern int http_main(int argc, char **argv, int do_init, void *main_out);
+extern int dacshttp_main(int argc, char **argv, int do_init, void *main_out);
 extern int rlink_main(int argc, char **argv, int do_init, void *main_out);
 extern int sslclient_main(int argc, char **argv, int do_init, void *main_out);
 
