@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -50,10 +50,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2013\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: aclcheck.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: aclcheck.c 2643 2013-02-27 19:59:24Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -83,6 +83,16 @@ use(void)
   exit(1);
 }
 
+static char *
+acl_index_pathname(Vfs_handle *h)
+{
+  char *p;
+
+  p = ds_xprintf("%s/%s", h->sd->uri_str, ACL_INDEX_FILENAME);
+
+  return(p);
+}
+
 /*
  *
  */
@@ -511,6 +521,23 @@ acl_map_compar(const void *ap, const void *bp)
   return(strcmp(a->path, b->path));
 }
 
+/*
+ * Return 1 if the index file exists, 0 if it does not, or -1 if the test failed.
+ */
+int
+exists_acl_index(Vfs_handle *h)
+{
+  int st;
+
+  st = vfs_exists(h, ACL_INDEX_FILENAME);
+  if (st == -1) {
+	log_msg((LOG_ERROR_LEVEL, "Error testing ACL index"));
+	return(-1);
+  }
+
+  return(st);
+}
+
 /*
  * Read and parse the index file, and sort it by priority.
  */
@@ -521,9 +548,9 @@ read_acl_index(Vfs_handle *h)
   Acl_index *ad;
 
   ad = NULL;
-  log_msg((LOG_DEBUG_LEVEL, "Reading index file \"%s\"", ACL_INDEX_FILENAME));
+  log_msg((LOG_DEBUG_LEVEL, "Reading index file \"%s\"", acl_index_pathname(h)));
   if (vfs_get(h, ACL_INDEX_FILENAME, (void *) &buf, NULL) == -1) {
-	log_msg((LOG_ERROR_LEVEL, "Error reading ACL index "));
+	log_msg((LOG_ERROR_LEVEL, "Error reading ACL index"));
 	return(NULL);
   }
 
@@ -790,7 +817,7 @@ convert_acl_index(char *item_type)
 	goto done;
 
   if (vfs_put(h, ACL_INDEX_FILENAME, ds_buf(&dir), ds_len(&dir) - 1) == -1) {
-	fprintf(stderr, "Could not write %s\n", ACL_INDEX_FILENAME);
+	fprintf(stderr, "Could not write %s\n", acl_index_pathname(h));
 	n = -1;
 	goto done;
   }
@@ -799,8 +826,7 @@ convert_acl_index(char *item_type)
  
   if (h != NULL) {
 	if (n == -1) {
-	  fprintf(stderr, "Could not write ACL index: %s/%s\n",
-			  h->sd->uri_str, ACL_INDEX_FILENAME);
+	  fprintf(stderr, "Could not write ACL index: %s\n", acl_index_pathname(h));
 	  if (h->error_msg != NULL)
 		fprintf(stderr, "%s\n", h->error_msg);
 	}
@@ -968,7 +994,7 @@ build_acl_index(char *item_type)
 	goto done;
 
   if (vfs_put(h, ACL_INDEX_FILENAME, ds_buf(&dir), ds_len(&dir) - 1) == -1) {
-	fprintf(stderr, "Could not write %s\n", ACL_INDEX_FILENAME);
+	fprintf(stderr, "Could not write %s\n", acl_index_pathname(h));
 	n = -1;
 	goto done;
   }
@@ -977,8 +1003,7 @@ build_acl_index(char *item_type)
  
   if (h != NULL) {
 	if (n == -1) {
-	  fprintf(stderr, "Could not write ACL index: %s/%s\n",
-			  h->sd->uri_str, ACL_INDEX_FILENAME);
+	  fprintf(stderr, "Could not write ACL index: %s\n", acl_index_pathname(h));
 	  if (h->error_msg != NULL)
 		fprintf(stderr, "%s\n", h->error_msg);
 	}
@@ -989,7 +1014,8 @@ build_acl_index(char *item_type)
 }
 
 /*
- * Check an ACL obtained from the store.
+ * Check ACL NAME, obtained from store H.
+ * Return 0 if successful, -1 if the check failed.
  */
 static int
 check(Vfs_handle *h, char *name)
@@ -998,12 +1024,12 @@ check(Vfs_handle *h, char *name)
   Parse_xml_error err;
 
   if (verbose_level)
-	log_msg((LOG_INFO_LEVEL, "Checking: %s/%s ...",
-			 h->sd->naming_context, name));
+	log_msg((LOG_INFO_LEVEL, "Checking: %s/%s ...", h->sd->naming_context, name));
 
   if (vfs_get(h, name, (void **) &buf, NULL) == -1) {
 	if (verbose_level)
 	  log_msg((LOG_ERROR_LEVEL, " %s", h->error_msg));
+
 	return(-1);
   }
 
@@ -1015,6 +1041,7 @@ check(Vfs_handle *h, char *name)
 	else
 	  log_msg((LOG_ERROR_LEVEL, "%s: %s", name, err.mesg));
 	free(buf);
+
 	return(-1);
   }
 
@@ -1026,10 +1053,11 @@ check(Vfs_handle *h, char *name)
 }
 
 /*
- * Check all ACLs in the store.
+ * Check all ACLs in the store that are currently indexed.
+ * Return the number of ACLs if successful, or -1 if any check failed.
  */
 static int
-check_all_acls(Vfs_handle *h)
+check_indexed_acls(Vfs_handle *h)
 {
   int i, n;
   Acl_index *ad;
@@ -1038,12 +1066,11 @@ check_all_acls(Vfs_handle *h)
   if ((ad = read_acl_index(h)) == NULL) {
 	log_msg((LOG_ERROR_LEVEL, "Could not get ACL list"));
 	if (h->error_msg != NULL)
-	  log_msg((LOG_ERROR_LEVEL, "%s: %s", h->sd->uri_str, h->error_msg));
+	  log_msg((LOG_ERROR_LEVEL, "%s: %s", acl_index_pathname(h), h->error_msg));
 	return(-1);
   }
 
-  n = dsvec_len(ad->acl_map);
-  if (n == 0)
+  if ((n = dsvec_len(ad->acl_map)) == 0)
 	return(0);
 
   amp = (Acl_map **) dsvec_base(ad->acl_map);
@@ -1098,37 +1125,60 @@ show_acl_item_type(FILE *fp, char *item_type, int long_form)
   return(0);
 }
 
+/*
+ * Check all ACLs (if FILES is NULL) associated with ITEM_TYPE, or only those in FILES
+ * (a vector terminated by a NULL element).
+ * Return 0 if successful, or -1 if an error occurred.
+ * XXX FILES should probably be a Dsvec
+ */
 static int
-check_acl_item_type(char *item_type, char **files)
+check_acl_item_type(char *item_type, char **files, int require_index)
 {
-  int i;
+  int i, rc, have_index;
   Vfs_directive *vd;
   Vfs_handle *h;
 
   if ((vd = vfs_lookup_item_type(item_type)) == NULL)
-	return(1);
+	return(-1);
 
   if (vfs_open(vd, &h) == -1)
 	return(-1);
 
-  if (files == NULL || files[0] == NULL)
-	check_all_acls(h);
+  have_index = exists_acl_index(h);
+  if (require_index) {
+	if (have_index == 0) {
+	  log_msg((LOG_ERROR_LEVEL, "Index does not exist: %s", acl_index_pathname(h)));
+	  return(-1);
+	}
+	if (have_index == -1) {
+	  log_msg((LOG_ERROR_LEVEL, "Cannot access index: %s", acl_index_pathname(h)));
+	  return(-1);
+	}
+  }
+
+  rc = 0;
+  if (files == NULL || files[0] == NULL) {
+	if (have_index) {
+	  if (check_indexed_acls(h) == -1)
+		rc = -1;
+	}
+  }
   else {
-	i = 0;
-	while (files[i] != NULL) {
-	  check(h, files[i]);
-	  i++;
+	for  (i = 0; files[i] != NULL; i++) {
+	  if (check(h, files[i]) == -1)
+		rc = -1;
 	}
   }
 
   vfs_close(h);
-  return(0);
+  return(rc);
 }
 
 int
 dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 {
-  int do_rebuild, i, long_form, n, st;
+  int do_build, i, long_form, n, st;
+  int saw_build_flag, saw_nobuild_flag;
   int saw_acls_vfs, saw_dacs_acls_vfs;
   char *errmsg;
 
@@ -1142,8 +1192,9 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
   i = 1;
 
   /* It's probably wise to always do this unless it's suppressed. */
-  do_rebuild = 1;
+  do_build = 1;
   saw_acls_vfs = saw_dacs_acls_vfs = 0;
+  saw_build_flag = saw_nobuild_flag = 0;
 
   while (i < argc) {
 	if (streq(argv[i], "--"))
@@ -1151,12 +1202,12 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 
 	if (streq(argv[i], "-convert")) {
 	  /* Convert from the old (non-indexed) format. */
-	  if ((n = convert_acl_index("acls")) == -1) {
-		fprintf(stderr, "Error converting \"acls\" ACLs\n");
+	  if ((n = convert_acl_index(ITEM_TYPE_ACLS)) == -1) {
+		fprintf(stderr, "Error converting \"%s\" ACLs\n", ITEM_TYPE_ACLS);
 		return(-1);
 	  }
 	  if (!quiet_flag)
-		fprintf(stderr, "Converted \"acls\": %d rules\n", n);
+		fprintf(stderr, "Converted \"%s\": %d rules\n", ITEM_TYPE_ACLS, n);
 
 #ifdef NOTDEF
 	  /*
@@ -1164,12 +1215,12 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 	   * releases that use the new format ship with their standard set already
 	   * converted, conversion of the standard set should be unnecessary.
 	   */
-	  if ((n = convert_acl_index("dacs_acls")) == -1) {
-		fprintf(stderr, "Error converting \"dacs_acls\" ACLs\n");
+	  if ((n = convert_acl_index(ITEM_TYPE_DACS_ACLS)) == -1) {
+		fprintf(stderr, "Error converting \"%s\" ACLs\n", ITEM_TYPE_DACS_ACLS);
 		return(-1);
 	  }
 	  if (!quiet_flag)
-		fprintf(stderr, "Converted \"dacs_acls\": %d rules\n", n);
+		fprintf(stderr, "Converted \"%s\": %d rules\n", ITEM_TYPE_DACS_ACLS, n);
 #endif
 
 	  return(0);
@@ -1297,18 +1348,18 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		return(-1);
 	  }
 
-	  show_acl_item_type(stdout, "acls", long_form);
-	  show_acl_item_type(stdout, "dacs_acls", long_form);
+	  show_acl_item_type(stdout, ITEM_TYPE_ACLS, long_form);
+	  show_acl_item_type(stdout, ITEM_TYPE_DACS_ACLS, long_form);
 
 	  return(0);
 	}
 
 	if (streq(argv[i], "-build")) {
-	  do_rebuild = 1;
+	  saw_build_flag = 1;
 	  i++;
 	}
 	else if (streq(argv[i], "-nobuild")) {
-	  do_rebuild = 0;
+	  saw_nobuild_flag = 1;
 	  i++;
 	}
 	else if (streq(argv[i], "-vfs")) {
@@ -1319,7 +1370,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		return(-1);
 	  }
 
-	  /* A VFS spec */
+	  /* A VFS spec... */
 	  if ((vd = vfs_uri_parse(argv[i], NULL)) == NULL) {
 		log_msg((LOG_ERROR_LEVEL, "Invalid vfs_uri"));
 		return(-1);
@@ -1327,9 +1378,9 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 	  conf_set_directive(var_ns_lookup_kwv(dacs_conf->conf_var_ns, "Conf"),
 						 "VFS", argv[i]);
 
-	  if (streq(vd->item_type, "acls"))
+	  if (streq(vd->item_type, ITEM_TYPE_ACLS))
 		saw_acls_vfs = 1;
-	  else if (streq(vd->item_type, "dacs_acls"))
+	  else if (streq(vd->item_type, ITEM_TYPE_DACS_ACLS))
 		saw_dacs_acls_vfs = 1;
 
 	  i++;
@@ -1361,65 +1412,116 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 	  break;
   }
 
-  if (i < argc) {
-	/* Check one or more ACLs in the virtual filestore. */
-	if ((st = check_acl_item_type("acls", &argv[i])) == -1) {
-	  log_msg((LOG_ERROR_LEVEL, "Error processing item_type \"acls\""));
-	  return(-1);
-	}
-	if (st == 1)
-	  log_msg((LOG_WARN_LEVEL, "No configuration for item_type \"acls\""));
-
-	if ((st = check_acl_item_type("dacs_acls", &argv[i])) == -1) {
-	  log_msg((LOG_ERROR_LEVEL, "Error processing item_type \"dacs_acls\""));
-	  return(-1);
-	}
-  }
-  else {
-	/* Check all ACLs... */
-	if ((st = check_acl_item_type("acls", NULL)) == -1) {
-	  log_msg((LOG_ERROR_LEVEL, "Error processing item_type \"acls\""));
-	  return(-1);
-	}
-	if (st == 1)
-	  log_msg((LOG_WARN_LEVEL, "No configuration for item_type \"acls\""));
-
-	if ((st = check_acl_item_type("dacs_acls", NULL)) == -1) {
-	  log_msg((LOG_ERROR_LEVEL, "Error processing item_type \"dacs_acls\""));
-	  return(-1);
-	}
+  if (saw_build_flag && saw_nobuild_flag) {
+	use();
+	/*NOTREACHED*/
   }
 
-  if (check_revocations() == -1)
-	return(-1);
+  if (saw_nobuild_flag)
+	do_build = 0;
+  else if (saw_build_flag)
+	do_build = 1;
 
-  if (do_rebuild) {
+  if (do_build) {
 	int n;
 
 	if (saw_acls_vfs || !saw_dacs_acls_vfs) {
 	  /* Build or rebuild indexes. */
-	  if ((n = build_acl_index("acls")) == -1) {
-		fprintf(stderr, "Error building \"acls\" ACLs\n");
+	  if ((n = build_acl_index(ITEM_TYPE_ACLS)) == -1) {
+		fprintf(stderr, "Error building \"%s\" ACLs\n", ITEM_TYPE_ACLS);
 		return(-1);
 	  }
 
 	  if (!quiet_flag)
-		fprintf(stderr, "Built index for \"acls\": %d rule%s\n",
-				n, (n == 1) ? "" : "s");
+		fprintf(stderr, "Built index for \"%s\": %d rule%s\n",
+				ITEM_TYPE_ACLS, n, (n == 1) ? "" : "s");
 	}
 
 	if (saw_dacs_acls_vfs || !saw_acls_vfs) {
-	  if ((n = build_acl_index("dacs_acls")) == -1) {
-		fprintf(stderr, "Error building \"dacs_acls\" ACLs\n");
+	  if ((n = build_acl_index(ITEM_TYPE_DACS_ACLS)) == -1) {
+		fprintf(stderr, "Error building \"%s\" ACLs\n", ITEM_TYPE_DACS_ACLS);
 		return(-1);
 	  }
 
 	  if (!quiet_flag)
-		fprintf(stderr, "Built index for \"dacs_acls\": %d rule%s\n",
-				n, (n == 1) ? "" : "s");
+		fprintf(stderr, "Built index for \"%s\": %d rule%s\n",
+				ITEM_TYPE_DACS_ACLS, n, (n == 1) ? "" : "s");
 	}
   }
 
+  /*
+   * Check ACLs
+   * If there are any non-flag arguments, they are the names of ACL files to
+   * validate; otherwise, check all ACL files.
+   */
+  if (i < argc) {
+	/* Check one or more ACL files in both of the virtual filestores. */
+	if (!dacs_no_conf_flag || saw_acls_vfs) {
+	  if ((st = check_acl_item_type(ITEM_TYPE_ACLS, &argv[i], 0)) == -1) {
+		log_msg((LOG_ERROR_LEVEL, "Error processing item_type \"%s\"",
+				 ITEM_TYPE_ACLS));
+		return(-1);
+	  }
+	  if (st == 1)
+		log_msg((LOG_WARN_LEVEL, "No config for item_type \"%s\"",
+				 ITEM_TYPE_ACLS));
+	}
+	else if (!quiet_flag)
+	  log_msg((LOG_WARN_LEVEL, "Ignoring undefined item_type \"%s\"",
+			   ITEM_TYPE_ACLS));
+
+	if (!dacs_no_conf_flag || saw_dacs_acls_vfs) {
+	  if ((st = check_acl_item_type(ITEM_TYPE_DACS_ACLS, &argv[i], 0)) == -1) {
+		log_msg((LOG_ERROR_LEVEL, "Error processing item_type \"%s\"",
+				 ITEM_TYPE_DACS_ACLS));
+		return(-1);
+	  }
+	  if (st == 1)
+		log_msg((LOG_WARN_LEVEL, "No config for item_type \"%s\"",
+				 ITEM_TYPE_DACS_ACLS));
+	}
+	else if (!quiet_flag)
+	  log_msg((LOG_WARN_LEVEL, "Ignoring undefined item_type \"%s\"",
+			   ITEM_TYPE_DACS_ACLS));
+  }
+  else {
+	/*
+	 * Check all ACLs that have been indexed.
+	 */
+	if (!dacs_no_conf_flag || saw_acls_vfs) {
+	  if ((st = check_acl_item_type(ITEM_TYPE_ACLS, NULL, !saw_build_flag))
+		  == -1) {
+		log_msg((LOG_ERROR_LEVEL, "Error processing item_type \"%s\"",
+				 ITEM_TYPE_ACLS));
+		return(-1);
+	  }
+	  if (st == 1)
+		log_msg((LOG_WARN_LEVEL, "No config for item_type \"%s\"",
+				 ITEM_TYPE_ACLS));
+	}
+	else if (!quiet_flag)
+	  log_msg((LOG_WARN_LEVEL, "Ignoring undefined item_type \"%s\"",
+			   ITEM_TYPE_ACLS));
+
+	if (!dacs_no_conf_flag || saw_dacs_acls_vfs) {
+	  if ((st = check_acl_item_type(ITEM_TYPE_DACS_ACLS, NULL,
+									!saw_build_flag)) == -1) {
+		log_msg((LOG_ERROR_LEVEL, "Error processing item_type \"%s\"",
+				 ITEM_TYPE_DACS_ACLS));
+		return(-1);
+	  }
+	  if (st == 1)
+		log_msg((LOG_WARN_LEVEL, "No config for item_type \"%s\"",
+				 ITEM_TYPE_DACS_ACLS));
+	}
+	else if (!quiet_flag)
+	  log_msg((LOG_WARN_LEVEL, "Ignoring undefined item_type \"%s\"",
+			   ITEM_TYPE_DACS_ACLS));
+  }
+
+  if (check_revocations() == -1)
+	return(-1);
+
   if (nchecks == 0)
 	log_msg((LOG_INFO_LEVEL, "No ACL files were checked"));
   else if (verbose_level)
