@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2013\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: acs.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: acs.c 2646 2013-02-27 20:10:40Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -72,8 +72,8 @@ acs_xml_html(FILE *fp, Acl_rule *acl)
 static int proxy_pass_authorization_flag = 0;
 static int pass_env_flag = 1;
 
-static int proxy_static_flag = 0;
 static int proxy_exec_flag = 0;
+static int proxy_static_flag = 0;
 
 typedef struct DACS_acs_args {
   int check_only;
@@ -339,6 +339,7 @@ main(int argc, char **argv)
   Credentials *credentials, *selected;
   Cookie *cookies;
   DACS_acs_args dacs_acs_args;
+  DACS_app_type app_type;
   Ds ds;
   Kwv *args, *kwv, *proxy_kwv;
   Kwv_pair *v;
@@ -364,9 +365,22 @@ main(int argc, char **argv)
   emitted_activity_cookie = 0;
   activity = NULL;
 
+  acs_init_result(&result);
+
   xargc = argc;
   xargv = argv;
-  if (dacs_init(DACS_ACS, &argc, &argv, &kwv, &errmsg) == -1) {
+
+  /*
+   * The test mode is a special case.  It (currently, at least) requires no
+   * configuration.  If "-test" is not the only flag it will still be
+   * recognized later, but normal configuration processing will be performed.
+   */
+  if (argc == 2 && streq(argv[1], "-test"))
+	app_type = DACS_STANDALONE;
+  else
+	app_type = DACS_ACS;
+
+  if (dacs_init(app_type, &argc, &argv, &kwv, &errmsg) == -1) {
 	char *denial_msg, *uri_path;
 	Acs_error_handler *handler;
 	Http_auth *auth;
@@ -794,7 +808,7 @@ main(int argc, char **argv)
 #ifdef NOTDEF
 	  proxy_static_flag = 1;
 #else
-	  /* Not currently supported */
+	  /* Not currently supported. */
 	  proxy_static_flag = 0;
 #endif
 	}
@@ -802,10 +816,20 @@ main(int argc, char **argv)
 #ifdef NOTDEF
 	  proxy_exec_flag = 1;
 #else
-	  /* Not currently supported */
+	  /* Not currently supported. */
 	  proxy_exec_flag = 0;
 #endif
 	}
+	else if (streq(argv[i], "-test")) {
+	  if (uri_pattern_match_selftest() != 0) {
+		fprintf(stderr, "Failed!\n");
+		exit(1);
+	  }
+
+	  fprintf(stderr, "All URI pattern matching tests succeeded!\n");
+	  exit(0);
+	  /*NOTREACHED*/
+	}
 	else {
 	  log_msg((LOG_ERROR_LEVEL, "Unrecognized argument: %s", argv[i]));
 	  errmsg = "Bad command line argument";
@@ -829,6 +853,12 @@ main(int argc, char **argv)
 	goto fail;
   }
 
+  if (dacs_no_conf_flag) {
+	errmsg = "No configuration has been specified";
+	error_occurred = 1;
+	goto fail;
+  }
+
   /*
    * Our standard input comes from mod_auth_dacs.
    * It is a series of "name=value\n" strings.
@@ -914,7 +944,7 @@ main(int argc, char **argv)
 	goto fail;
   }
   log_msg((LOG_INFO_LEVEL, "SERVICE_URI=%s", service_uri));
-  dacs_service_uri = service_uri;
+  set_dacs_service_uri(service_uri);
   if ((p = current_uri_no_query(NULL)) != NULL)
 	log_msg((LOG_INFO_LEVEL, "Current URI (sans query)=%s", p));
 
