@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2011
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2011\n\
+"Copyright (c) 2003-2013\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: conf.c 2510 2011-09-23 17:06:22Z brachman $";
+  "$Id: conf.c 2642 2013-02-27 19:48:45Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -1374,7 +1374,7 @@ domain_equiv(const char *subdomain, const char *domain)
   if (domain[0] != '*' || domain[1] != '.')
 	return(strcaseeq(subdomain, domain));
 
-  return(match_domain_names(subdomain, domain + 2));
+  return(match_subdomain_name(subdomain, domain + 2));
 }
 
 static int
