@@ -47,7 +47,7 @@ static const char copyright[] =
 "Copyright (c) 2003-2012\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: dacspasswd.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: dacspasswd.c 2610 2012-10-19 18:02:36Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -634,7 +634,7 @@ pw_parse_constraints(char *constraint, Pw_constraints *conf)
   if (p == NULL || *p == '\0' || strcaseeq(p, "none"))
 	goto done;
 
-  if ((dsv = strsplit_re(p, "[ \t]*,[ \t]*", 0, &errmsg)) == NULL)
+  if ((dsv = strsplit_re(p, "[ \t]*,[ \t]*", 0, 0, &errmsg)) == NULL)
 	return(-1);
 
   for (i = 0; i < dsvec_len(dsv); i++) {
