@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -77,10 +77,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2013\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: ds.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: ds.c 2640 2013-02-27 19:41:06Z brachman $";
 #endif
 
 /*
@@ -2384,6 +2384,8 @@ new_dsvec(size_t size)
 
 /*
  * Initialize for allocating a vector of items, each SIZE bytes in size.
+ * XXX there should also be a dsvec_reinit() to free/reinitialize an
+ * existing Dsvec more efficiently than a dsvec_free()/dsvec_init() combo
  */
 Dsvec *
 dsvec_init(Dsvec *odsv, size_t size)
@@ -2547,14 +2549,15 @@ dsvec_load_lines(Dsvec *dsv, char *pathname)
 }
 
 /*
- * Create a copy of OLD_DSV, starting at pointer offset START and extending for
- * LEN pointers (if LEN is -1, copy until the end of OLD_DSV).
+ * Create a copy of OLD_DSV, starting at pointer offset START (zero or greater)
+ * and extending for LEN pointers (if LEN is -1, copy until the end of OLD_DSV).
+ * Return NULL if the selected region of OLD_DSV is invalid.
  */
 Dsvec *
 dsvec_subset(Dsvec *odsv, Dsvec *old_dsv, unsigned int start, int len)
 {
-  int new_nused;
-  unsigned int offset;
+  int nptrs, new_nused;
+  unsigned int start_offset;
   size_t need;
   Dsvec *dsv;
 
@@ -2562,7 +2565,18 @@ dsvec_subset(Dsvec *odsv, Dsvec *old_dsv, unsigned int start, int len)
   if (old_dsv == NULL)
 	return(NULL);
 
-  if ((new_nused = old_dsv->nused - start) <= 0)
+  /* Validate the specified length. */
+  if (len <= 0) {
+	if (len == -1)
+	  nptrs = old_dsv->nused;
+	else
+	  return(NULL);
+  }
+  else
+	nptrs = len;
+
+  /* Validate the starting point and the extent of the region. */
+  if (start < 0 || (start + nptrs) > old_dsv->nused)
 	return(NULL);
 
   if (odsv == NULL) {
@@ -2579,8 +2593,8 @@ dsvec_subset(Dsvec *odsv, Dsvec *old_dsv, unsigned int start, int len)
   dsv->exact_flag = old_dsv->exact_flag;
   dsv->el_size = old_dsv->el_size;
 
-  dsv->nused = new_nused;
-  dsv->nalloc = new_nused;
+  dsv->nused = nptrs;
+  dsv->nalloc = nptrs;
   dsv->nused_limit = old_dsv->nused_limit;
   dsv->malloc = old_dsv->malloc;
   dsv->free = old_dsv->free;
@@ -2591,8 +2605,8 @@ dsvec_subset(Dsvec *odsv, Dsvec *old_dsv, unsigned int start, int len)
 	return(NULL);
   }
 
-  offset = dsv->el_size * (old_dsv->nused - dsv->nused);
-  memcpy(dsv->ptr, old_dsv->ptr + offset, need);
+  start_offset = dsv->el_size * start;
+  memcpy(dsv->ptr, old_dsv->ptr + start_offset, need);
 
   return(dsv);
 }
@@ -2649,6 +2663,23 @@ dsvec_ptr_index(Dsvec *dsv, unsigned int ind)
   return(dsv->ptr[ind]);
 }
 
+/*
+ * Like dsvec_ptr_index(), but "reversed" in that RIND is the offset from
+ * the end rather than start.  So RIND==0 means the last element,
+ * RIND==1 means the next to last element, and so on.  This can be useful
+ * when comparing two vectors from their tails.
+ */
+void *
+dsvec_ptr_rindex(Dsvec *dsv, unsigned int rind)
+{
+  int offset;
+
+  offset = dsv->nused - rind - 1;
+  if (offset < 0)
+	return(NULL);
+  return(dsv->ptr[offset]);
+}
+
 /*
  * Prepare DSV so that it can hold at least NELEMENTS more elements.
  * Return 0 if ok, -1 if it fails.
