@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -49,10 +49,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2013\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: http.c 2588 2012-03-15 20:01:54Z brachman $";
+  "$Id: http.c 2642 2013-02-27 19:48:45Z brachman $";
 #endif
 
 #include "dacs_config.h"
@@ -108,6 +108,7 @@ int is_http_prog = 0;
 
 static int use_authorization_header = 0;
 static char *user_agent_string = NULL;
+static int verbose_flag = 0;
 
 /*
  * Add an HTTP parameter called NAME with value VALUE (which may be NULL,
@@ -554,6 +555,50 @@ decode_hexpair(char *str, int *val)
 }
 #endif
 
+/*
+ * Given PATH, the path part of an encoded URL (with or without a leading
+ * '/'), break it into its slash delimited components.  Terminate the list
+ * with a NULL entry.
+ * Return NULL if a decoding error occurs.
+ */
+Dsvec *
+uri_path_parse(char *path)
+{
+  char *p, *s;
+  Ds *ds;
+  Dsvec *dsv;
+
+  dsv = dsvec_init(NULL, sizeof(Ds *));
+
+  if (path == NULL) {
+	dsvec_add_ptr(dsv, NULL);
+	return(dsv);
+  }
+	
+  if (*path == '/')
+	path++;
+
+  for (p = s = strdup(path); *p != '\0'; p++) {
+	if (*p == '/') {
+	  *p++ = '\0';
+	  if (url_decode(s, NULL, &ds) == NULL)
+		return(NULL);
+	  dsvec_add_ptr(dsv, ds);
+	  s = p;
+	}
+  }
+
+  if (*s != '\0') {
+	if (url_decode(s, NULL, &ds) == NULL)
+	  return(NULL);
+	dsvec_add_ptr(dsv, ds);
+  }
+
+  dsvec_add_ptr(dsv, NULL);
+
+  return(dsv);
+}
+
 /*
  * Extract the path, query, and fragment part of URI (all optionally) but
  * do NOT decode them.
@@ -770,7 +815,7 @@ uri_is_valid_path_part(char *part)
 }
 
 /*
- * Parse the path component of a URI (the query and/or fragment component
+ * Break the path component of a URI (the query and/or fragment component
  * having already been removed) into individual segments, then assemble the
  * URL decoded version of the path.
  *
@@ -787,7 +832,7 @@ uri_is_valid_path_part(char *part)
  * URI->path to the path.
  */
 int
-uri_parse_path(Uri *uri, char *str)
+uri_make_path(Uri *uri, char *str)
 {
   int i;
   char *decoded_part;
@@ -882,7 +927,7 @@ uri_parse_hier_part(Uri *uri, char *str, int has_authority)
   else if ((q = strchr(p, (int) '#')) != NULL)
 	*q = '\0';
 
-  if (uri_parse_path(uri, p) == -1)
+  if (uri_make_path(uri, p) == -1)
 	return(NULL);
 
   if (q == NULL)
@@ -1273,8 +1318,9 @@ send_request(Http *h)
 	pre_path = (*p == '/') ? "" : "/";
 
   ds_init(&request);
-  ds_asprintf(&request, "%s %s%s HTTP/%s\r\n",
+  ds_asprintf(&request, "%s %s%s%s HTTP/%s\r\n",
 			  http_method_to_string(h->method),
+			  (h->proxy.hostname != NULL) ? uri_scheme_authority(h->uri) : "",
 			  pre_path, p,
 			  HTTP_VERSION);
   /* (14.23) */
@@ -1362,6 +1408,8 @@ send_request(Http *h)
 
   log_msg(((Log_level) (LOG_TRACE_LEVEL | LOG_SENSITIVE_FLAG),
 		   "Request:\n%s", ds_buf(&request)));
+  if (verbose_flag > 1)
+	fprintf(stderr, "Request:\n%s", ds_buf(&request));
 
   rc = write_to_server(h, ds_buf(&request), ds_len(&request) - 1);
 
@@ -2049,9 +2097,9 @@ usage(void)
 }
 
 int
-http_main(int argc, char **argv, int do_init, void *main_out)
+dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 {
-  int i, have_method, prompt_user, nredirects, rc, saw_ssl_flag, vflag;
+  int i, have_method, prompt_user, nredirects, rc, saw_ssl_flag;
   int include_headers;
   char *proto, *redirect_uri, *request_uri;
   Dsvec *ssl_args, *ssl_flags, *post_params;
@@ -2074,7 +2122,7 @@ http_main(int argc, char **argv, int do_init, void *main_out)
   log_set_level(ld, LOG_WARN_LEVEL);
   log_set_desc(ld, LOG_ENABLED);
   log_set_flags(ld, LOG_MATCH_NORMAL | LOG_MATCH_SENSITIVE);
-  vflag = 0;
+  verbose_flag = 0;
   vfp = stderr;
 
   if (argc < 2) {
@@ -2179,7 +2227,7 @@ http_main(int argc, char **argv, int do_init, void *main_out)
 	}
 
 	if (streq(argv[i], "-v")) {
-	  vflag++;
+	  verbose_flag++;
 	  continue;
 	}
 
@@ -2383,9 +2431,9 @@ http_main(int argc, char **argv, int do_init, void *main_out)
   if ((request_uri = argv[i]) == NULL)
 	goto fail;
 
-  if (vflag == 1)
+  if (verbose_flag == 1)
 	log_set_level(ld, LOG_DEBUG_LEVEL);
-  else if (vflag > 1)
+  else if (verbose_flag > 1)
 	log_set_level(ld, LOG_TRACE_LEVEL);
 
  http_invoke:
@@ -2424,7 +2472,7 @@ http_main(int argc, char **argv, int do_init, void *main_out)
 	else
 	  dsvec_add_ptr(ssl_args, uri->host);
 
-	if (vflag)
+	if (verbose_flag)
 	  fprintf(stderr, "Using SSL: %s\n", strjoin(ssl_args, " "));
   }
 
@@ -2434,7 +2482,7 @@ http_main(int argc, char **argv, int do_init, void *main_out)
 	http_set_ssl_params(h, ssl_args);
   }
 
-  if (vflag) {
+  if (verbose_flag) {
 	Uri_query_arg *r;
 
 	fprintf(stderr, "Method: %s\n", http_method_to_string(h->method));
@@ -2468,10 +2516,10 @@ http_main(int argc, char **argv, int do_init, void *main_out)
 	return(-1);
   }
 
-  if (vflag || include_headers)
+  if (verbose_flag || include_headers)
 	fprintf(vfp, "Status-Line: %s\n", h->status_line);
 
-  if ((vflag  || include_headers) && h->response_headers != NULL) {
+  if ((verbose_flag  || include_headers) && h->response_headers != NULL) {
     char *p;
   
     for (i = 0; (p = dsvec_ptr(h->response_headers, i, char *)) != NULL; i++)
@@ -2483,17 +2531,27 @@ http_main(int argc, char **argv, int do_init, void *main_out)
   redirect_uri = kwv_lookup_value(h->response_kwv, "Location");
 
   if (redirect_uri == NULL || prompt_user || h->method != HTTP_HEAD_METHOD) {
+	FILE *ofp;
+
 	/* Read and display the response. */
-	if (vflag || include_headers) {
+	if (verbose_flag || include_headers) {
 	  fprintf(vfp, "\n");
 	  fflush(vfp);
 	}
 
+	if (h->status_code >= 400)
+	  ofp = stderr;
+	else
+	  ofp = stdout;
+
 	ds_init(&response);
 	while ((rc = http_read_response_body(h, &response)) > 0) {
-	  fwrite(ds_buf(&response), 1, ds_len(&response), stdout);
+	  fwrite(ds_buf(&response), 1, ds_len(&response), ofp);
 	  ds_reset(&response);
 	}
+	if (h->status_code >= 400)
+	  fprintf(ofp, " (Status-Code %d)\n", h->status_code);
+	fflush(ofp);
 
 	if (rc == -1) {
 	  fprintf(stderr, "Error reading response\n");
@@ -2506,7 +2564,7 @@ http_main(int argc, char **argv, int do_init, void *main_out)
 	Ds ds;
 	Uri *ruri;
 
-	if (vflag)
+	if (verbose_flag)
 	  fprintf(stderr, "Received redirect to: %s\n", redirect_uri);
 
 	/* RFC 2616 S10.3 */
@@ -2548,7 +2606,7 @@ http_main(int argc, char **argv, int do_init, void *main_out)
 		  goto done;
 		}
 	  }
-	  if (vflag)
+	  if (verbose_flag)
 		fprintf(stderr, "Will redirect to: %s\n", request_uri);
 	}
 	else
@@ -2583,6 +2641,11 @@ http_main(int argc, char **argv, int do_init, void *main_out)
 	goto http_invoke;
   }
 
+  if (h->status_code == 407) {
+	/* Server requires proxy authentication. */
+	fprintf(stderr, "Server requires proxy authentication\n");
+  }
+
  done:
 
   http_close(h);
@@ -2597,9 +2660,9 @@ main(int argc, char **argv)
 {
   int rc;
   extern int is_http_prog;
-
+ 
   is_http_prog = 1;
-  if ((rc = http_main(argc, argv, 1, NULL)) == 0)
+  if ((rc = dacshttp_main(argc, argv, 1, NULL)) == 0)
 	exit(0);
 
   exit(1);
