@@ -37,7 +37,7 @@ static const char copyright[] =
 "Copyright (c) 2003-2012\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: mime.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: mime.c 2610 2012-10-19 18:02:36Z brachman $";
 #endif
 
 #ifdef DSSLIB
@@ -1608,7 +1608,7 @@ mime_get_file_types(char *pathname)
 	if (*p == '\0' || *p == '#')
 	  continue;
 
-	if ((fields = strsplit_re(p, "[ \t]+", 0, &errmsg)) == NULL)
+	if ((fields = strsplit_re(p, "[ \t]+", 0, 0, &errmsg)) == NULL)
 	  return(NULL);
 
 	mft = ALLOC(Mime_file_type);
