@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2011
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -12,10 +12,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2011\n\
+"Copyright (c) 2003-2013\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: simple_api.c 2528 2011-09-23 21:54:05Z brachman $";
+  "$Id: simple_api.c 2642 2013-02-27 19:48:45Z brachman $";
 #endif
 
 #include <pwd.h>
@@ -953,20 +953,25 @@ set_ident_config(int *argcp, char **argv, Kwv *kwv_dacs, Kwv *kwv_conf,
  * The Conf namespace context will come from dacs_conf->conf_var_ns.
  */
 Acs_environment *
-init_env(char *object, Kwv *kwv_conf, Kwv *kwv_dacs)
+init_env(char *object, Uri *uri, Kwv *kwv_conf, Kwv *kwv_dacs)
 {
   char *url;
   Acs_environment *env;
   Var_ns *conf_var_ns;
 
-  url = ds_xprintf("file://%s", object);
-
   env = acs_new_env(NULL);
-  env->url = url;
+  env->uri = uri;
+  if (*object == '/')
+	env->url = ds_xprintf("file://%s", object);
+  else
+	env->url = strdup(object);
+
   conf_var_ns = var_ns_create("Conf", kwv_conf);
+
   if (kwv_dacs == NULL)
 	kwv_dacs = kwv_init(10);
   kwv_merge(kwv_dacs, kwv_dacsoptions, KWV_REPLACE_DUPS);
+
   var_ns_new(&env->namespaces, "DACS", kwv_dacs);
 
   dacs_conf = new_dacs_conf(NULL, NULL, NULL, conf_vartab, conf_var_ns);
@@ -1050,7 +1055,7 @@ check_object_access(char *rules, char *groups, char *idents,
 	putenv(str);
   }
 
-  if (uri->host != NULL)
+  if (uri->host != NULL && uri->host[0] != '\0')
 	host = uri->host;
   else {
 	if (hostname != NULL)
@@ -1156,7 +1161,7 @@ check_object_access(char *rules, char *groups, char *idents,
 	  putenv(str);
 	}
 
-	env = init_env(object, kwv_conf, kwv_dacs);
+	env = init_env(object, uri, kwv_conf, kwv_dacs);
 	if (acs_init_env(NULL, kwv_args, uri->path, credentials, env) == -1)
 	  return(-1);
 
